import { ExecutionEngine } from './executionEngine';
import { TraceOptions } from '../common/models/engineTraceOptions.model';
/**
 * Represents an abstract EngineTask with a reference to the ExecutionEngine.
 *
 * @abstract
 */
export declare abstract class EngineTask {
    engine: ExecutionEngine;
}
/**
 * A class decorator that enhances a class with execution engine capabilities.
 * @param options - Configuration options for the execution engine.
 */
export declare function engine(options?: {
    id: string;
}): ClassDecorator;
/**
 * A method decorator that enables tracing for the decorated method.
 *
 * @param {TraceOptions<Array<any>, O> | TraceOptions<Array<any>, O>['trace']} [options] - Optional tracing options.
 * @returns {Function} - A decorator function.
 */
export declare function run<O>(options?: TraceOptions<Array<any>, O> | TraceOptions<Array<any>, O>['trace']): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => PropertyDescriptor;
//# sourceMappingURL=executionEngineDecorators.d.ts.map